@@ -1,8 +1,10 @@
-import time
-import configargparse
-import re
-from pathlib import Path
 import logging
+import pathlib
+import re
+import time
+
+import configargparse
+
 import redgrease.formatting
 import redgrease.loader
 
@@ -18,7 +20,7 @@ args.add_argument(
     "directories",
     metavar="dir_path",
     nargs="+",
-    type=Path,
+    type=pathlib.Path,
     help="One or more directories containing Redis Gears scripts to watch",
 )
 
@@ -132,12 +134,6 @@ args.add_argument(
 )
 
 
-def blah(
-    apanapansson: int = 2, bepanbepansdotter: str = "wut teh hellz", some_value=None
-):
-    ...
-
-
 config = args.parse_args()
 
 redgrease.formatting.initialize_logger(config=config.log_config)
@@ -169,7 +165,6 @@ def main():
             while running:
                 time.sleep(1)
         except KeyboardInterrupt:
-            running = False
             log.warning("Interrupted by user. Ending!")
         finally:
             loader.stop()
