@@ -1,15 +1,16 @@
 import logging
-from typing import Any, List, Mapping, Optional, Union
+import os.path
+from typing import Any, Iterable, List, Mapping, Optional, Union
 
 import redis
-import redis.client
-import redis.exceptions
 
 import redgrease.config
 import redgrease.data
+import redgrease.gears
+import redgrease.reader
+import redgrease.requirements
 from redgrease.utils import (
     CaseInsensitiveDict,
-    as_is,
     bool_ok,
     dict_of,
     list_parser,
@@ -151,12 +152,16 @@ class Gears:
         return self.redis.execute_command("RG.INFOCLUSTER")
 
     def pyexecute(
-        self, function_string: str, unblocking=False, requirements: List[str] = None
+        self,
+        gear_function: Union[str, redgrease.gears.ClosedGearFunction] = "",
+        unblocking=False,
+        requirements: Iterable[Union[str, redgrease.requirements.Requirement]] = None,
+        enforce_redgrease: redgrease.requirements.PackageOption = False,
     ):
         """Execute Python code
 
         Args:
-            function_string (str): Serialized Gears Python function
+            gear_function (str): Serialized Gears Python function
 
             unblocking (bool, optional): Execute function without waiting for
             it to finish, before returnining.
@@ -180,15 +185,49 @@ class Gears:
             A simple 'OK' string is returned if the function has no output
             (i.e. it doesn't consist of any functions with the run action).
         """
+        requirements = set(requirements if requirements else [])
+
+        if isinstance(gear_function, redgrease.gears.GearFunction):
+            # If the input is a GearFunction, we get the requirements from it,
+            # and ensure that redgrease is included
+            requirements = requirements.union(gear_function.requirements)
+            enforce_redgrease = True
+
+            if isinstance(gear_function, redgrease.gears.PartialGearFunction):
+                # If the function isn't closed with either 'run' or 'register'
+                # we assume it is meant to be closed with a 'run'
+                gear_function = gear_function.run()
+
+            function_string = redgrease.data.seralize_gear_function(gear_function)
+
+        elif os.path.exists(gear_function):
+            # If the gear function is a fle path,
+            # then we load the contents of the file
+            with open(gear_function) as script_file:
+                function_string = script_file.read()
+
+        else:
+            # Otherwise we default to the string version of the function
+            function_string = str(gear_function)
+
         params = []
         if unblocking:
             params.append("UNBLOCKING")
 
-        if requirements is not None:
-            params.append("REQUIREMENTS")
-            params += requirements
+        requirements = redgrease.requirements.resolve_requirements(
+            requirements, enforce_redgrease=enforce_redgrease
+        )
 
-        return self.redis.execute_command("RG.PYEXECUTE", function_string, *params)
+        if requirements:
+            params.append("REQUIREMENTS")
+            params += list(map(str, requirements))
+
+        return self.redis.execute_command(
+            "RG.PYEXECUTE",
+            function_string,
+            *params,
+            pickled_results=isinstance(gear_function, redgrease.gears.GearFunction)
+        )
 
     def pystats(self) -> redgrease.data.PyStats:
         """Get memory usage statisticy from the Python interpreter
@@ -265,14 +304,14 @@ class Redis(redis.Redis):
             "RG.DUMPEXECUTIONS": list_parser(redgrease.data.Registration.from_redis),
             "RG.DUMPREGISTRATIONS": list_parser(redgrease.data.Registration.from_redis),
             "RG.GETEXECUTION": redgrease.data.ExecutionPlan.parse,
-            "RG.GETRESULTS": as_is,
-            "RG.GETRESULTSBLOCKING": as_is,
+            "RG.GETRESULTS": redgrease.data.parse_execute_response,
+            "RG.GETRESULTSBLOCKING": redgrease.data.parse_execute_response,
             "RG.INFOCLUSTER": redgrease.data.ClusterInfo.parse,
             "RG.PYEXECUTE": redgrease.data.parse_execute_response,
             "RG.PYSTATS": redgrease.data.PyStats.from_redis,
             "RG.PYDUMPREQS": list_parser(redgrease.data.PyRequirementInfo.from_redis),
             "RG.REFRESHCLUSTER": bool_ok,
-            "RG.TRIGGER": as_is,
+            "RG.TRIGGER": redgrease.data.parse_trigger_response,
             "RG.UNREGISTER": bool_ok,
         },
     }
