@@ -1,22 +1,21 @@
+import functools
 import types
-from functools import lru_cache
 
-import redgrease
-from redgrease.client import RedisGears
+import redgrease.client
+import redgrease.runtime
 
 
-def runtime_execute_command(self, *args, **options):
+def runtime_execute_command(self, command_name, *args, **options):
     "Execute a command in local Gear runtime and return a parsed response"
-    command_name = args[0]
-    response = redgrease.execute(*args)
+    response = redgrease.runtime.execute(command_name, *args)
     if command_name in self.response_callbacks:
         return self.response_callbacks[command_name](response, **options)
     return response
 
 
-@lru_cache()
+@functools.lru_cache()
 def get_runtime_client():
-    runtime_client = RedisGears(connection_pool=...)
+    runtime_client = redgrease.client.RedisGears(connection_pool=...)
     runtime_client.execute_command = types.MethodType(
         runtime_execute_command, runtime_client
     )
