@@ -1,8 +1,12 @@
 import ast
-from typing import Any, Dict, Iterable, List, Union
+import sys
+from typing import Any, Dict, Generic, Iterable, List, Optional, TypeVar, Union
 
 import attr
+import cloudpickle
+import wrapt
 
+import redgrease.gears
 from redgrease.utils import (
     REnum,
     bool_ok,
@@ -10,28 +14,12 @@ from redgrease.utils import (
     safe_bool,
     safe_str,
     str_if_bytes,
+    to_bytes,
     to_dict,
     to_kwargs,
 )
 
-
-@attr.s(auto_attribs=True, frozen=True)
-class Execution:
-    results: List
-    errors: List
-
-
-# TODO: Should this be in utils?
-# TODO: Rethink how execution redponses should be handlde
-def parse_execute_response(response):
-    if bool_ok(response):
-        return True
-    elif isinstance(response, list) and len(response) == 2:
-        return Execution(*response)
-    elif isinstance(response, bytes):
-        return Execution(ExecID.parse(response), [])
-    else:
-        return response
+T = TypeVar("T")
 
 
 @attr.s(frozen=True, auto_attribs=True)
@@ -64,6 +52,92 @@ class ExecID:
         return ExecID(shard_id=shard_id, sequence=sequence)
 
 
+class ExecutionResult(wrapt.ObjectProxy, Generic[T]):
+    def __init__(self, value: T, errors: Optional[List] = None):
+        wrapt.ObjectProxy.__init__(self, value)
+        self._self_errors: Optional[List] = errors
+
+    @property
+    def value(self):
+        return self.__wrapped__
+
+    @property
+    def errors(self):
+        return self._self_errors
+
+    def __repr__(self) -> str:
+        cls_nm = "ExecutionResult"
+        val_typ_nm = self.__class__.__name__
+        if self._self_errors:
+            err = f", errors={repr(self._self_errors)}"
+        else:
+            err = ""
+
+        return f"{cls_nm}[{val_typ_nm}]({self.__wrapped__.__repr__()}{err})"
+
+    def __iter__(self):
+        if hasattr(self.value, "__iter__"):
+            return iter(self.value)
+        else:
+            return iter([] if self.value is None else [self.value])
+
+    def __len__(self) -> int:
+        if hasattr(self.value, "__len__") and not isinstance(self.value, (str, bytes)):
+            return len(self.value)
+        else:
+            return 0 if self.value is None else 1
+
+    def __eq__(self, other):
+        if not isinstance(self.value, list) and isinstance(other, list):
+            return [self.value] == other
+        else:
+            return self.value == other
+
+    def __getitem__(self, *args, **kwargs):
+        if hasattr(self.value, "__getitem__"):
+            return self.value.__getitem__(*args, **kwargs)
+        else:
+            if args and args[0] == 0:
+                return self.value
+            else:
+                raise TypeError(f"{self} is not subscriptable")
+
+    def __contains__(self, val) -> bool:
+        if hasattr(self.value, "__contains__"):
+            return val in self.value
+        else:
+            return self.value == val
+
+    def __bytes__(self) -> bytes:
+        return to_bytes(self.value)
+
+
+def parse_execute_response(response, pickled_results=False) -> ExecutionResult:
+    if bool_ok(response):
+        return ExecutionResult(True)
+    elif isinstance(response, list) and len(response) == 2:
+        result, errors = response
+        if isinstance(result, list):
+            if pickled_results:
+                result = [cloudpickle.loads(value) for value in result]
+            if len(result) == 1:
+                result = result[0]
+        return ExecutionResult(result, errors=errors)
+    elif isinstance(response, bytes):
+        return ExecutionResult(ExecID.parse(response))
+    else:
+        return ExecutionResult(response)
+
+
+def parse_trigger_response(response, pickled_results=False) -> ExecutionResult:
+    if pickled_results:
+        response = [cloudpickle.loads(value) for value in response]
+
+    if len(response) == 1:
+        response = response[0]
+    return ExecutionResult(response)
+
+
 class ExecutionStatus(REnum):
     created = b"created"
     running = b"running"
@@ -204,3 +278,38 @@ class PyRequirementInfo(RedisObject):
         if isinstance(wheels, bytes)
         else [safe_str(wheel) for wheel in wheels]
     )
+
+
+def deseralize_gear_function(
+    serialized_gear: str, python_version: str
+) -> redgrease.gears.GearFunction:
+    try:
+        return cloudpickle.loads(serialized_gear)
+    except Exception as err:
+        import sys
+
+        def pystr(pyver):
+            return "Python %s.%s" % pyver
+
+        runtime_python_version = sys.version_info[:2]
+        function_python_version = ast.literal_eval(str(python_version))[:2]
+        if runtime_python_version != function_python_version:
+            raise SystemError(
+                f"{pystr(runtime_python_version)} runtime cannot execute "
+                f"Gears functions created in {function_python_version}. "
+                "Only matching Python versions are supported"
+            ) from err
+        raise
+
+
+def seralize_gear_function(gear_function: redgrease.gears.ClosedGearFunction) -> str:
+    return f"""
+import redgrease.data
+import redgrease.runtime
+
+gear_function = redgrease.data.deseralize_gear_function(
+    {cloudpickle.dumps(gear_function, protocol=4)},
+    python_version={tuple(sys.version_info)},
+)
+redgrease.runtime.run(gear_function, GearsBuilder)
+"""
